--***************************************************************
-- 
-- load from central config file
-- 
-- version 1.3 by mogli (biedens)
-- 2014/08/06
--
--***************************************************************

source(Utils.getFilename("mogliBase.lua", g_currentModDirectory))
mrGearboxMogliLoader = mogliBase11.newSubclass()

source(Utils.getFilename("mrGearboxMogli.lua", g_currentModDirectory))

for _,funcName in pairs({ "delete", 
													"mouseEvent", 
													"keyEvent",
													"readStream", 
													"writeStream", 
													"update", 
													"updateTick", 
													"readUpdateStream", 
													"writeUpdateStream", 
													"draw", 
													"getSaveAttributesAndNodes", 
													"loadFromAttributesAndNodes",
													"realSetSpeedLevelLimit" }) do
	if  mrGearboxMogli  ~= nil and mrGearboxMogli[funcName] ~= nil and type(mrGearboxMogli[funcName])  == "function" then
		mrGearboxMogliLoader[funcName] = function( self, ... )
			if type(self)=="table" then
				local state, result = nil, nil
				if self.mrGbMLGearbox1 then
					state, result = pcall(mrGearboxMogli[funcName], self, ...)
					if state then
						return result
					else
						print("Error 3: "..tostring(result))
					end
				end
			end
			
			if funcName == "loadFromAttributesAndNodes" then
				return BaseMission.VEHICLE_LOAD_OK
			end
		end		
	end	
end

function mrGearboxMogliLoader.prerequisitesPresent(specializations) 
	return SpecializationUtil.hasSpecialization(RealisticMotorized, specializations) 
end 

function mrGearboxMogliLoader:load(xmlFile) 
	
	self.mrGbMLGearbox1       = false
	self.mrGbMLConfigFileName = Utils.removeModDirectory(self.configFileName);
	
	if self.mrGbMLConfigFileName == "" then
		print("Error -1: "..tostring(self.customEnvironment).." "..tostring(self.mrGbMLConfigFileName))
		return
	end
		
	local state, message = pcall( mrGearboxMogliLoader.loadGearboxMogli, self )
	if not state then
		print("Error 3 loading mrGearboxMogliLoader: "..tostring(message)) 
		self.mrGbMLGearbox1 = false
	end

end 

function mrGearboxMogliLoader.initXmlFiles()

	if mrGearboxMogliLoader.initXmlFilesDone then 
		return 
	end
	
	mrGearboxMogliLoader.initXmlFilesDone = true
		
	mrGearboxMogliLoader.configInt = {}
	mrGearboxMogliLoader.configExt = {}
	
	local file1 = mrGearboxMogliLoader.baseDirectory.."zzzMrGearboxAddonConfig.xml"
	print(file1)
	if fileExists(file1) then	
		local xmlFile = loadXMLFile( "vehicles", file1, "vehicles" )
		mrGearboxMogliLoader.xmlFileInt = xmlFile
		
		i=0
		while true do
			local baseName       = string.format("vehicles.vehicle(%d)", i)
			local j=0
			local configFileName = getXMLString(xmlFile, baseName .. "#configFileName")
			while true do
				local entry = { xmlName = baseName }

				if configFileName ~= nil then
					entry.configFileName = configFileName
				else
					local modName = getXMLString(xmlFile, string.format( "%s.configFile(%d)#modName", entry.xmlName, j ))
					local xmlName = getXMLString(xmlFile, string.format( "%s.configFile(%d)#xmlName", entry.xmlName, j ))
					if modName == nil and xmlName == nil then
						break
					elseif modName ~= nil and xmlName ~= nil then
						entry.configFileName = modName .. "/" .. xmlName
					else
						entry.configFileName = xmlName
					end
				end
				if entry.configFileName ~= nil then
					print(string.format( "zzzMrGearboxAddon: found configuration for %s", entry.configFileName ))				
					entry.configFileName = string.lower( entry.configFileName )
					table.insert( mrGearboxMogliLoader.configInt, entry )
				end
				j = j + 1
				if configFileName ~= nil then
					break
				end
			end
			if j<=0 then
				break
			end
			i = i + 1
		end
	end

	local file2 = mrGearboxMogliLoader.modsDirectory.."zzzMrGearboxAddonConfig.xml"
	print(file2)
	if fileExists(file2) then	
		local xmlFile = loadXMLFile( "vehicles", file2, "vehicles" )
		mrGearboxMogliLoader.xmlFileExt = xmlFile
		
		i=0
		while true do
			local baseName       = string.format("vehicles.vehicle(%d)", i)
			local configFileName = getXMLString(xmlFile, baseName .. "#configFileName")
			local j              = 0
			while true do
				local entry = { xmlName = baseName }

				if configFileName ~= nil then
					entry.configFileName = configFileName
				else
					local modName = getXMLString(xmlFile, string.format( "%s.configFile(%d)#modName", entry.xmlName, j ))
					local xmlName = getXMLString(xmlFile, string.format( "%s.configFile(%d)#xmlName", entry.xmlName, j ))
					if modName == nil and xmlName == nil then
						break
					elseif modName ~= nil and xmlName ~= nil then
						entry.configFileName = modName .. "/" .. xmlName
					else
						entry.configFileName = xmlName
					end
				end
				if entry.configFileName ~= nil then
					print(string.format( "zzzMrGearboxAddon: found external configuration for %s", entry.configFileName ))				
					entry.configFileName = string.lower( entry.configFileName )
					table.insert( mrGearboxMogliLoader.configExt, entry )
				end
				j = j + 1
				if configFileName ~= nil then
					break
				end
			end
			if j<=0 then
				break
			end
			i = i + 1
		end
	end
end

function mrGearboxMogliLoader.testXMLProperty( xmlFile, name )
	if name == nil then
		return true
	end
	local s = getXMLString( xmlFile, name )
	if s == nil or s == "" then
		return false
  end
	return true
end

function mrGearboxMogliLoader:loadGeneric( func, tagName, propName1, propName2 )
	mrGearboxMogliLoader.initXmlFiles()
	
	local xmlFile
	local configFileName = string.lower( self.mrGbMLConfigFileName )
	
	xmlFile = mrGearboxMogliLoader.xmlFileExt
	
	for _,entry in pairs(mrGearboxMogliLoader.configExt) do
		if      entry.configFileName == configFileName
				and ( ( propName1 ==nil and propName2 ==nil )
					 or ( propName1 ~=nil and propName2 ==nil 
						and mrGearboxMogliLoader.testXMLProperty(xmlFile, entry.xmlName..propName1 ) )
					 or ( propName1 ==nil and propName2 ~=nil 
						and mrGearboxMogliLoader.testXMLProperty(xmlFile, entry.xmlName..propName2 ) )
					 or ( propName1 ~=nil and propName2 ~=nil
					  and ( mrGearboxMogliLoader.testXMLProperty(xmlFile, entry.xmlName..propName1 ) 
							 or mrGearboxMogliLoader.testXMLProperty(xmlFile, entry.xmlName..propName2 ) ) ) ) then
			local state, message = pcall( func, self, xmlFile, entry.xmlName )	
			if state and message then
				print(string.format( "zzzMrGearboxAddon: %s inserted into %s", tagName, self.mrGbMLConfigFileName ))
				return true
			elseif not state then
				print("Error 4 loading mrGearboxMogliLoader: "..tostring(message)) 
			end
		end
	end
	
	xmlFile = mrGearboxMogliLoader.xmlFileInt
	for _,entry in pairs(mrGearboxMogliLoader.configInt) do
		if      entry.configFileName == configFileName
				and ( ( propName1 ==nil and propName2 ==nil )
					 or ( propName1 ~=nil and propName2 ==nil 
						and mrGearboxMogliLoader.testXMLProperty(xmlFile, entry.xmlName..propName1 ) )
					 or ( propName1 ==nil and propName2 ~=nil 
						and mrGearboxMogliLoader.testXMLProperty(xmlFile, entry.xmlName..propName2 ) )
					 or ( propName1 ~=nil and propName2 ~=nil
					  and ( mrGearboxMogliLoader.testXMLProperty(xmlFile, entry.xmlName..propName1 ) 
							 or mrGearboxMogliLoader.testXMLProperty(xmlFile, entry.xmlName..propName2 ) ) ) ) then
			local state, message = pcall( func, self, xmlFile, entry.xmlName )	
			if state and message then
				print(string.format( "zzzMrGearboxAddon: %s inserted into %s", tagName, self.mrGbMLConfigFileName ))
				return true
			elseif not state then
				print("Error 5 loading mrGearboxMogliLoader: "..tostring(message)) 
			end
		end
	end

	print(string.format( "zzzMrGearboxAddon: no configuration found for inserting %s into %s", tagName, self.mrGbMLConfigFileName ))
	return false
end


function mrGearboxMogliLoader:loadGearboxMogli()
	self.mrGbMLGearbox1 = mrGearboxMogliLoader.loadGeneric( self, mrGearboxMogliLoader.loadGearboxMogli2, "gearboxMogli",  ".gearboxMogli.gears.gear(0)#speed" )
end

function mrGearboxMogliLoader:loadGearboxMogli2(xmlFile,baseName)	
	mrGearboxMogli.initFromXml( self, xmlFile, baseName .. ".gearboxMogli" )
	return true
end

